SystemOrganization addCategory: #MonticelloConfigurations!
(PackageInfo named: 'MonticelloConfigurations') postscript: '"below, add code to be run after the loading of this package"
MCConfiguration upgradeIsMerge: Preferences upgradeIsMerge.'!

MCTool subclass: #MCConfigurationBrowser
	instanceVariableNames: 'configuration dependencyIndex repositoryIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCConfigurationBrowser commentStamp: 'dtl 5/10/2010 21:48' prior: 0!
A MCConfigurationBrowser displays an MCConfiguration, and edits the configuration to add or remove package dependencies and repository specifications. It allows a configuration to be stored in a repository or posted to an update stream.!

!MCConfigurationBrowser class methodsFor: 'class initialization' stamp: 'bf 3/21/2005 19:46'!
initialize
	TheWorldMenu registerOpenCommand: { 'Monticello Configurations' . { self . #open }. 'Monticello Configuration Browser' }.! !

!MCConfigurationBrowser class methodsFor: 'opening' stamp: 'bf 3/21/2005 19:50'!
open
	^self new show! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 3/23/2005 22:08'!
addDependency
	(self pickWorkingCopiesSatisfying: [:each | (self includesPackage: each package) not])
		do: [:wc |
			wc ancestors isEmpty
				ifTrue: [self inform: 'You must save ', wc packageName, ' first!!
Skipping this package']
				ifFalse: [
					self dependencies add: (MCVersionDependency
						package: wc package
						info: wc ancestors first)]].
	self changed: #dependencyList; changed: #description! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'cmm 4/22/2011 17:11'!
addRepository
	(self pickRepositorySatisfying:
		[ : ea | (self repositories includes: ea) not ]) ifNotNil:
		[ : repo | repo class supportsConfigurations ifFalse: [ ^ self inform: 'Not all of these repositories support MCConfigurations.' ].
		self repositories add: repo.
		self changed: #repositoryList ]! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'bf 10/15/2014 10:35'!
buttonSpecs
	^ #(('Add' addDependency 'Add a dependency')
		('Update' updateMenu 'Update dependencies')
		('Install' installMenu 'Load/Merge/Upgrade into image')
		('Up' up 'Move item up in list' canMoveUp)
		('Down' down 'Move item down in list' canMoveDown)
		('Remove' remove 'Remove item' canRemove)
		('Store' store 'store configuration')
		('Post' post 'Post this configuration to an update stream')
		)! !

!MCConfigurationBrowser methodsFor: 'testing' stamp: 'bf 3/23/2005 20:44'!
canMoveDown
	^self index between: 1 and: self maxIndex - 1 ! !

!MCConfigurationBrowser methodsFor: 'testing' stamp: 'bf 3/23/2005 20:44'!
canMoveUp
	^self index > 1! !

!MCConfigurationBrowser methodsFor: 'testing' stamp: 'bf 3/23/2005 20:45'!
canRemove
	^self index > 0! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 5/27/2005 19:54'!
changedButtons
	self changed: #canMoveDown.
	self changed: #canMoveUp.
	self changed: #canRemove.! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 20:55'!
changedList
	self dependencyIndex > 0 ifTrue: [^self changed: #dependencyList].
	self repositoryIndex > 0 ifTrue: [^self changed: #repositoryList].
	self error: 'nothing selected'! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 4/19/2005 17:36'!
checkDependencies
	^self checkModified and: [self checkMissing]! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'cmm 2/27/2011 16:39'!
checkMissing
	| missing |
	missing := (self dependencies collect:
		[ : ea | ea versionInfo name ]) asSet.
	self repositories do:
		[ : eachRepository | eachRepository cacheAllFileNamesDuring:
			[ missing copy do:
				[ : eachVersionName | (eachRepository includesVersionNamed: eachVersionName) ifTrue: [ missing remove: eachVersionName ] ] ] ].
	^ missing isEmpty or:
		[ self selectDependency: missing anyOne.
		self confirm:
			(String streamContents:
				[ : strm | strm
					 nextPutAll: 'No repository found for' ;
					 cr.
				missing do:
					[ : r | strm
						 nextPutAll: r ;
						 cr ].
				strm nextPutAll: 'Do you still want to store?' ]) ]! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 4/19/2005 17:37'!
checkModified
	| modified |
	modified := self dependencies select: [:dep |
		dep isFulfilled and: [dep package workingCopy modified]].
	
	^modified isEmpty or: [
		self selectDependency: modified anyOne.
		self confirm: (String streamContents: [:strm |
			strm nextPutAll: 'These packages are modified:'; cr.
			modified do: [:dep | strm nextPutAll: dep package name; cr].
			strm nextPutAll: 'Do you still want to store?'])]
	! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'cmm 4/22/2011 17:12'!
checkRepositories
	| bad |
	bad := self repositories reject: [:repo | repo class supportsConfigurations ].
	^bad isEmpty or: [
		self selectRepository: bad first.
		self inform: (String streamContents: [:strm |
			strm nextPutAll: 'Please remove these repositories:'; cr.
			bad do: [:r | strm nextPutAll: r description; cr].
			strm nextPutAll: '(only HTTP repositories are supported)']).
		false].
! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'bf 3/24/2005 00:47'!
checkRepositoryTemplates
	"unused for now - we only do HTTP"
	| bad |
	bad := self repositories select: [:repo | repo creationTemplate isNil].
	^bad isEmpty or: [
		self selectRepository: bad first.
		self inform: (String streamContents: [:strm |
			strm nextPutAll: 'Creation template missing for'; cr.
			bad do: [:r | strm nextPutAll: r description; cr].
			strm nextPutAll: 'Please fill in the details first!!']).
		false].
! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 3/21/2005 16:03'!
configuration
	^configuration ifNil: [configuration := MCConfiguration new]! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 3/21/2005 14:56'!
configuration: aConfiguration
	configuration := aConfiguration! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'bf 4/19/2005 16:51'!
defaultExtent
	^ 350@500! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 3/21/2005 16:35'!
dependencies
	^self configuration dependencies
! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 4/19/2005 16:02'!
dependencies: aCollection
	self configuration dependencies: aCollection.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 17:56'!
dependencyIndex
	^dependencyIndex ifNil: [0]! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 5/27/2005 19:55'!
dependencyIndex: anInteger
	dependencyIndex := anInteger.
	dependencyIndex > 0
		ifTrue: [self repositoryIndex: 0].
	self changed: #dependencyIndex; changed: #description.
	self changedButtons.! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 4/19/2005 17:28'!
dependencyList
	^self dependencies collect: [:dep | 
		Text string: dep versionInfo name
			attributes: (Array streamContents: [:attr |
				dep isFulfilledByAncestors
					ifFalse: [attr nextPut: TextEmphasis bold]
					ifTrue: [dep isCurrent ifFalse: [attr nextPut: TextEmphasis italic]].
			])]
! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'bf 10/16/2012 10:56'!
dependencyMenu: aMenu
	self fillMenu: aMenu fromSpecs: #(('add new dependency...' addDependency)).
	self selectedDependency ifNotNil: [
		self fillMenu: aMenu fromSpecs: #(
			addLine
			('remove this dependency' remove)
			('update this dependency from image' updateSelectedDependencyFromImage)
			('update this dependency from repositories' updateSelectedDependencyFromRepositories)
		)].
	^aMenu! !

!MCConfigurationBrowser methodsFor: 'description' stamp: 'eem 5/10/2014 07:19'!
description
	self selectedDependency ifNotNil:
		[:dep |
		^ ('Package: ', dep package name, String cr, dep versionInfo summary) asText].
	self selectedRepository ifNotNil:
		[:repo |
		^repo creationTemplate
			ifNotNil: [repo creationTemplate asText]
			ifNil: [repo asCreationTemplate asText addAttribute: TextColor red]].
	^('A configuration is a set of particular versions of packages.  These can be used to manage multiple dependencies amongst packages when an update requires changes to multiple packages.  One stores the current configuration and then modifies the various packages needing modification.  On load, the image will be updated to at least the versions in the current configuration, hence providing the support needed to load the new packages.\\To create a new configuration first load the most up-to-date configuration in your repository (e.g.',  MCHttpRepository trunkUrlString, '), open that repository in the Monticello browser, scroll down to the "update" package, select the first entry in the list on the right hand side and click Browse, which will open the configuration in a new MCConfigurationBrowser.  Then in the new MCConfigurationBrowser click Update, and choose "update all from image" from the pop-up menu.  Click Store to save back to the repository.  If required, one can add repositories to the browser to store the configuration in a different repository.') withCRs! !

!MCConfigurationBrowser methodsFor: 'description' stamp: 'ul 12/12/2009 14:10'!
description: aText

	self selectedRepository ifNotNil: [:repo | 
		| new | 
		new := MCRepository readFrom: aText asString.
		(new class = repo class 
			and: [new description = repo description])
				ifTrue: [
					repo creationTemplate: aText asString.
					self changed: #description]
				ifFalse: [
					self inform: 'This does not match the previous definition!!'
				]
	].

! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 3/23/2005 21:01'!
down
	self canMoveDown ifTrue: [
		self list swap: self index with: self index + 1.
		self index: self index + 1.
		self changedList.
	].
! !

!MCConfigurationBrowser methodsFor: 'testing' stamp: 'bf 3/21/2005 17:15'!
includesPackage: aPackage
	^self dependencies anySatisfy: [:each | each package = aPackage]! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 20:43'!
index
	^self dependencyIndex max: self repositoryIndex! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 21:00'!
index: anInteger
	self dependencyIndex > 0 ifTrue: [^self dependencyIndex: anInteger].
	self repositoryIndex > 0 ifTrue: [^self repositoryIndex: anInteger].
	anInteger > 0 ifTrue: [self error: 'cannot select']! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'fbs 4/23/2013 16:26'!
installMenu
	| action |
	action := UIManager default
		chooseFrom: #('load packages' 'merge packages' 'upgrade packages')
		values: #(#load #merge #upgrade).
	action ifNotNil: [self perform: action].! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 20:51'!
list
	self dependencyIndex > 0 ifTrue: [^self dependencies].
	self repositoryIndex > 0 ifTrue: [^self repositories].
	^#()! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 4/19/2005 17:42'!
load
	self configuration load.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 20:52'!
maxIndex
	^ self list size! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 4/19/2005 17:42'!
merge
	self configuration merge.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'ul 3/14/2011 17:21'!
pickName
	| name |
	name := UIManager default
		request: 'Name (.', self configuration writerClass extension, ' will be appended):'
		initialAnswer: self configuration suggestedNameOfNextVersion.
	^ name isEmpty ifFalse: [name]! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'bf 3/23/2005 21:11'!
pickRepository
	^self pickRepositorySatisfying: [:ea | true]
! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'ar 8/6/2009 18:23'!
pickRepositorySatisfying: aBlock
	| index list |
	list := MCRepositoryGroup default repositories select: aBlock.
	index := UIManager default chooseFrom: (list collect: [:ea | ea description])
		title: 'Repository:'.
	^ index = 0 ifFalse: [list at: index]! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'ar 8/6/2009 20:41'!
pickWorkingCopiesSatisfying: aBlock
	| copies item |
	copies := (MCWorkingCopy allManagers select: aBlock)
		asSortedCollection: [:a :b | a packageName <= b packageName].
	item := UIManager default chooseFrom: #('match ...'),(copies collect: [:ea | ea packageName]) lines: #(1) title: 'Package:'.
	item = 1 ifTrue: [
		| pattern |
		pattern := UIManager default request: 'Packages matching:' initialAnswer: '*'.
		^pattern isEmptyOrNil
			ifTrue: [#()]
			ifFalse: [
				(pattern includes: $*) ifFalse: [pattern := '*', pattern, '*'].
				copies select: [:ea | pattern match: ea packageName]]
	].
	^ item = 0
		ifTrue: [#()]
		ifFalse: [{copies at: item - 1}]! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 3/23/2005 21:05'!
remove
	self canRemove ifTrue: [
		self list removeAt: self index.
		self changedList.
		self updateIndex.
	].
! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'dtl 5/10/2010 21:01'!
removeRepository
	repositoryIndex > 0
		ifTrue: [self repositories removeAt: repositoryIndex.
			repositoryIndex := 0.
			self changed: #repositoryList]! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 3/23/2005 17:41'!
repositories
	^ self configuration repositories! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 3/23/2005 21:15'!
repositories: aCollection
	^self configuration repositories: aCollection
! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 17:57'!
repositoryIndex
	^repositoryIndex ifNil: [0]! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 5/27/2005 19:55'!
repositoryIndex: anInteger
	repositoryIndex := anInteger.
	repositoryIndex > 0
		ifTrue: [self dependencyIndex: 0].
	self changed: #repositoryIndex; changed: #description.
	self changedButtons.! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'bf 3/23/2005 21:15'!
repositoryList
	^self repositories collect: [:ea | ea description]
! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'dtl 5/10/2010 20:56'!
repositoryMenu: aMenu
	self fillMenu: aMenu fromSpecs: #(('add repository...' addRepository)).
	self selectedRepository ifNotNil: [
		self fillMenu: aMenu fromSpecs: #(('remove repository' removeRepository))].
	^aMenu
! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 23:16'!
selectDependency: aDependency
	self dependencyIndex: (self dependencies indexOf: aDependency)! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 23:15'!
selectRepository: aRepository
	self repositoryIndex: (self repositories indexOf: aRepository)! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 3/23/2005 17:56'!
selectedDependency
	^ self dependencies at: self dependencyIndex ifAbsent: []! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'ul 12/12/2009 14:10'!
selectedPackage
	^ self selectedDependency ifNotNil: [:dep | dep package]! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'bf 3/23/2005 17:58'!
selectedRepository
	^ self repositories at: self repositoryIndex ifAbsent: []! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:10'!
store
	(self checkRepositories and: [self checkDependencies]) ifFalse: [^self].
	self pickName ifNotNil: [:name |
		self configuration name: name.
		self pickRepository ifNotNil: [:repo |
			repo storeVersion: self configuration]].! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 3/23/2005 20:53'!
up
	self canMoveUp ifTrue: [
		self list swap: self index with: self index - 1.
		self index: self index - 1.
		self changedList.
	].! !

!MCConfigurationBrowser methodsFor: 'updating' stamp: 'bf 5/23/2005 17:44'!
updateFromImage
	self configuration updateFromImage.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'updating' stamp: 'bf 5/23/2005 17:44'!
updateFromRepositories
	self configuration updateFromRepositories.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 21:00'!
updateIndex
	self index > 0 ifTrue: [self index: (self index min: self maxIndex)]! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'fbs 4/23/2013 16:24'!
updateMenu
	| action |
	action := UIManager default
		chooseFrom: #('update all from image' 'update all from repositories')
		values: #(#updateFromImage #updateFromRepositories).
	action ifNotNil: [self perform: action].! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'nice 3/31/2012 22:48'!
updateSelectedDependencyFromImage
	self configuration updateFromImage: self dependencyIndex.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'nice 3/31/2012 22:48'!
updateSelectedDependencyFromRepositories
	self configuration updateFromRepositories: self dependencyIndex.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 4/19/2005 17:43'!
upgrade
	self configuration upgrade.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'mt 4/30/2015 13:34'!
widgetSpecs
	^ #(
		((buttonRow) (0 0 1 0) (0 0 0 defaultButtonPaneHeight))
		((listMorph:selection:menu: dependencyList dependencyIndex dependencyMenu:) (0 0 1 1) (0 defaultButtonPaneHeight 0 -180))
		((listMorph:selection:menu: repositoryList repositoryIndex repositoryMenu:) (0 1 1 1) (0 -180 0 -120))
		((textMorph: description) (0 1 1 1) (0 -120 0 0))
	 	)! !

!MCRepository class methodsFor: '*monticelloconfigurations' stamp: 'cmm 4/22/2011 16:52'!
supportsConfigurations
	^ false! !

RWBinaryOrTextStream subclass: #MCPseudoFileStream
	instanceVariableNames: 'localName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCPseudoFileStream commentStamp: '<historical>' prior: 0!
A pseudo file stream which can be used for updates.!

!MCPseudoFileStream methodsFor: 'accessing' stamp: 'ar 4/14/2005 19:54'!
localName
	^localName! !

!MCPseudoFileStream methodsFor: 'accessing' stamp: 'ar 4/14/2005 19:54'!
localName: aString
	localName := aString! !

MCWriter subclass: #MCMcmWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCMcmWriter commentStamp: 'dtl 5/10/2010 22:20' prior: 0!
An MCMcmWriter stores an MCConfiguration on a stream in the form of an array specification.
!

!MCMcmWriter class methodsFor: 'writing' stamp: 'stephaneducasse 2/4/2006 20:47'!
fileOut: aConfiguration on: aStream
	| inst |
	inst := self on: aStream.
	inst writeConfiguration: aConfiguration.
	inst close.
	
! !

!MCMcmWriter class methodsFor: 'accessing' stamp: 'bf 3/22/2005 10:49'!
readerClass
	^ MCMcmReader! !

!MCMcmWriter methodsFor: 'writing' stamp: 'bf 3/22/2005 18:00'!
close
	stream close! !

!MCMcmWriter methodsFor: 'writing' stamp: 'cmm 10/8/2013 10:44'!
writeConfiguration: aConfiguration

	stream nextPut: $(.

	aConfiguration repositories do: [:ea | 
		stream cr.
		stream nextPutAll: 'repository '.
		(MCConfiguration repositoryToArray: ea) printElementsOn: stream].

	aConfiguration dependencies do: [:ea | 
		stream cr.
		stream nextPutAll: 'dependency '.
		(MCConfiguration dependencyToArray: ea) printElementsOn: stream].

	stream nextPutAll: 'name '; nextPutAll: aConfiguration name printString.

	stream cr.
	stream nextPut: $).
	stream cr.! !

Object subclass: #MCConfiguration
	instanceVariableNames: 'name dependencies repositories log'
	classVariableNames: 'DefaultLog ExtraProgressInfo LogToFile UpgradeIsMerge'
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCConfiguration commentStamp: 'dtl 5/10/2010 23:03' prior: 0!
An MCConfiguration specifies the configuration of a set of related Monticello packages. It maintains an ordered list of package versions and a list of repositories in which the packages may be found.

An MCConfiguration may be filed out for storage as an array specification, and new instances can be created from a stored array specification.
!

!MCConfiguration class methodsFor: 'accessing' stamp: 'ar 5/27/2005 17:27'!
defaultLog
	"Answer the default configuration log"
	^DefaultLog! !

!MCConfiguration class methodsFor: 'accessing' stamp: 'ar 5/27/2005 17:27'!
defaultLog: aStream
	"Set the default configuration log"
	DefaultLog := aStream.! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 3/24/2005 01:43'!
dependencyFromArray: anArray
	^MCVersionDependency
		package: (MCPackage named: anArray first)
		info: (
			MCVersionInfo
			name: anArray second
			id: (UUID fromString: anArray third)
			message: nil
			date: nil
			time: nil
			author: nil
			ancestors: nil)! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 3/24/2005 01:44'!
dependencyToArray: aDependency
	^ {
		aDependency package name . 
		aDependency versionInfo name . 
		aDependency versionInfo id asString }! !

!MCConfiguration class methodsFor: 'preferences' stamp: 'ar 12/30/2009 17:30'!
extraProgressInfo
	"Answer true for additional progress info during load. 
	With the newly added MC down/upload operations this seems unnecessary
	but some people might disagree, so let's leave it as a preference right now"
	<preference: 'Extra Progress Info' 
		category: 'Monticello' 
		description: 'If true, additional progress information is displayed when loading MC configurations (i.e., during updates)' 
		type: #Boolean>
	^ExtraProgressInfo ifNil:[false]! !

!MCConfiguration class methodsFor: 'preferences' stamp: 'ar 12/30/2009 17:30'!
extraProgressInfo: aBool
	"Whether to display for additional progress info during load."
	ExtraProgressInfo := aBool.
! !

!MCConfiguration class methodsFor: 'instance creation' stamp: 'cmm 10/8/2013 10:45'!
fromArray: anArray
	| configuration |
	configuration := self new.
	anArray pairsDo: [:key :value |
		key = #repository
			ifTrue: [configuration repositories add: (self repositoryFromArray: value)].
		key = #dependency
			ifTrue: [configuration dependencies add: (self dependencyFromArray: value)].
		key = #name
			ifTrue: [configuration name: value].
	].
	^configuration! !

!MCConfiguration class methodsFor: 'preferences' stamp: 'ar 8/21/2009 21:18'!
logToFile
	"Whether to log configuration info to files by default.
	If true, logs to a file named after the configuration (config.nn.log).
	If false, logs to the transcript."
	<preference: 'Log config info to disk' 
		category: 'Monticello' 
		description: 'If true, configuration information (such as change logs) are logged to disk instead of the Transcript. The log file is named after the configuration map (config.nn.log)' 
		type: #Boolean>
	^LogToFile ifNil:[true].! !

!MCConfiguration class methodsFor: 'preferences' stamp: 'ar 8/21/2009 21:13'!
logToFile: aBool
	"Whether to log configuration info to files by default.
		MCConfiguration logToFile: true.
		MCConfiguration logToFile: false.
	"
	LogToFile := aBool! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 6/9/2005 14:25'!
repositoryFromArray: anArray
	^ MCRepositoryGroup default repositories
		detect: [:repo | repo description = anArray first]
		ifNone: [
			MCHttpRepository
				location: anArray first
				user: ''
				password: '']! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 3/24/2005 01:51'!
repositoryToArray: aRepository
	^ {aRepository description}! !

!MCConfiguration class methodsFor: 'preferences' stamp: 'fbs 12/31/2013 13:59'!
upgradeIsMerge
	"Answer true if you wish to merge upstream changes whenever you upgrade."
	<preference: 'Upgrade is merge'
		category: 'updates'
		description: 'When upgrading packages, use merge instead of load'
		type: #Boolean>
	^UpgradeIsMerge ifNil: [true]! !

!MCConfiguration class methodsFor: 'preferences' stamp: 'fbs 11/21/2013 09:03'!
upgradeIsMerge: aBoolean
	UpgradeIsMerge := aBoolean.! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 3/22/2005 22:09'!
browse
	(MCConfigurationBrowser new configuration: self) show! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/22/2012 18:02'!
cacheAllFileNamesDuring: aBlock
	^ (repositories
		inject: aBlock
		into: [ :innerBlock :repository |
			[ repository cacheAllFileNamesDuring: innerBlock ]
		]) value
	
! !

!MCConfiguration methodsFor: 'faking' stamp: 'bf 3/24/2005 01:19'!
changes
	^MCPatch operations: #()! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/21/2005 16:32'!
dependencies
	^dependencies ifNil: [dependencies := OrderedCollection new]! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/21/2005 18:40'!
dependencies: aCollection
	dependencies := aCollection! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 9/11/2013 17:16'!
depsSatisfying: selectBlock versionDo: verBlock displayingProgress: progressString 
	| count selectedVersions cleanWorkingCopies |
	self cacheAllFileNamesDuring: [
		self repositories do: [ :eachRepository |
			MCRepositoryGroup default addRepository: eachRepository ].
		"First, download selected versions"
		count := 0.
		selectedVersions := OrderedCollection new.
		self withProgress: progressString in: self dependencies do: [ :dep | | verName repo |
			verName := dep versionInfo name.
			self class extraProgressInfo ifTrue:
				[ ProgressNotification signal: '' extra: 'Downloading ' , verName ].
			repo := self repositories
				detect: [ :eachRepository | eachRepository includesVersionNamed: verName ]
				ifNone: 	[ self logError: 'Version ' , verName , ' not found in any repository'.
					self logError: 'Aborting'.
					^ count ].
			(selectBlock value: dep) ifTrue: [ | version |
				version := self versionNamed: verName for: dep from: repo.
				version ifNil: [ self logError: 'Could not download version ' , verName , ' from ' , repo description.
					self logError: 'Aborting'.
					^ count ].
				dep package workingCopy newRepositoryGroupIfDefault. "fix old working copies"
				dep package workingCopy repositoryGroup addRepository: repo.
				selectedVersions add: version]].
		"Then, process only those definitions that moved from one package to another, to avoid order dependence"
		cleanWorkingCopies := MCWorkingCopy allManagers reject: [ :wc | wc modified ].
		MCReorganizationPreloader preloadMovesBetween: selectedVersions.
		"Finally, load/merge selected versions"
		self withProgress: progressString in: selectedVersions do: [ :version |
			self logUpdate: version package with: version.
			self class extraProgressInfo ifTrue:
				[ ProgressNotification signal: '' extra: 'Installing ' , version info name ].
			verBlock value: version.
			count := count + 1 ].
		"Clean up packages made dirty by MCReorganizationPreloader"
		cleanWorkingCopies do: [ :wc | wc modified ifTrue: [wc checkModified] ].
	].
	^ count! !

!MCConfiguration methodsFor: 'private' stamp: 'cmm 3/5/2011 14:54'!
diffBaseFor: aDependency
	| wc |
	aDependency package hasWorkingCopy ifFalse: [^nil].
	wc := aDependency package workingCopy.
	wc ancestors ifEmpty: [^nil].
	^wc ancestors first versionName! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 18:22'!
fileName
	^ self name, '.', self writerClass extension
! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 3/22/2005 10:51'!
fileOutOn: aStream
	self writerClass fileOut: self on: aStream! !

!MCConfiguration methodsFor: 'faking' stamp: 'bf 3/24/2005 01:17'!
info
	^MCVersionInfo new! !

!MCConfiguration methodsFor: 'initialize' stamp: 'ar 5/27/2005 17:28'!
initialize
	super initialize.
	log := DefaultLog.! !

!MCConfiguration methodsFor: 'testing' stamp: 'bf 3/22/2005 22:56'!
isCacheable
	^false! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 4/22/2005 17:19'!
load
	^self depsSatisfying: [:dep | dep isCurrent not]
		versionDo: [:ver | ver load]
		displayingProgress: 'loading packages'
! !

!MCConfiguration methodsFor: 'accessing' stamp: 'mp 2/2/2010 20:38'!
log
	"Answer the receiver's log. If no log exist use the default log"
	
	^log ifNil: [
		(name notNil and: [ self class logToFile ]) ifFalse: [ 
			Transcript countOpenTranscripts = 0 ifTrue: [Transcript open].
			^Transcript ].
		self log: ((FileStream fileNamed: self logFileName) setToEnd; yourself).
		log ]! !

!MCConfiguration methodsFor: 'accessing' stamp: 'ar 4/28/2005 11:55'!
log: aStream
	log := aStream.! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 16:07'!
logError: aString
	self log
		cr; nextPutAll: 'ERROR: ';
		nextPutAll: aString; cr;
		flush.
! !

!MCConfiguration methodsFor: 'accessing' stamp: 'fbs 7/25/2013 07:13'!
logFileName

	^self name, '-', (FileDirectory localNameFor: Smalltalk imageName), '.log'
	! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 15:59'!
logUpdate: aPackage with: aVersion
	self log
		cr; nextPutAll: '========== ', aVersion info name, ' =========='; cr;
		cr; nextPutAll: aVersion info message asString; cr;
		flush.

	aPackage hasWorkingCopy ifFalse: [^self].

	aPackage workingCopy ancestors do: [:each |
		(aVersion info hasAncestor: each)
			ifTrue: [(aVersion info allAncestorsOnPathTo: each)
				do: [:ver | self log cr; nextPutAll: '>>> ', ver name, ' <<<'; cr;
							nextPutAll: ver message; cr; flush]]]! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 16:08'!
logWarning: aString
	self log
		cr; nextPutAll: 'WARNING: ';
		nextPutAll: aString; cr;
		flush.
! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 4/22/2005 17:19'!
merge
	^self depsSatisfying: [:dep | dep isFulfilledByAncestors not]
		versionDo: [:ver | ver merge]
		displayingProgress: 'merging packages'
! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 5/23/2005 14:47'!
mustMerge: aVersion
	"answer true if we have to do a full merge and false if we can simply load instead"
	
	| pkg wc current |
	(pkg := aVersion package) hasWorkingCopy ifFalse: [^false "no wc -> load"].
	(wc := pkg workingCopy) modified ifTrue: [^true "modified -> merge"].
	wc ancestors isEmpty ifTrue: [^true "no ancestor info -> merge"].
	current := wc ancestors first.
	(aVersion info hasAncestor: current) ifTrue: [^false "direct descendant of wc -> load"].
	"new branch -> merge"
	^true! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 18:23'!
name
	^name! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 18:23'!
name: aString
	name := aString! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/23/2005 17:35'!
repositories
	^repositories ifNil: [repositories := OrderedCollection new]! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/23/2005 17:36'!
repositories: aCollection
	repositories := aCollection! !

!MCConfiguration methodsFor: 'updating' stamp: 'fbs 11/23/2013 21:05'!
setSystemVersion
	"Set the current system version date to the latest date found in my configuration (or the associated working copy). Also set the highest update number to the sum of version numbers in my configuration."

	| versionNumbers versionDates |
	versionNumbers := self dependencies collect: [:d |
		(d versionInfo name copyAfterLast: $.) asInteger].
	versionDates := self dependencies collect: [:d |
		d versionInfo date
			ifNil: [d package workingCopy ancestors first date]].
	SystemVersion current
		date: versionDates max;
		highestUpdate: versionNumbers sum.! !

!MCConfiguration methodsFor: 'private' stamp: 'cmm 3/15/2011 14:03'!
suggestedNameOfNextVersion
	"Suggest a name for the next version of this configuration. The format is assumed to be name-authorInitials.version. Automatically increments the version, takes author initials from Utilities."
	^'{1}-{2}.{3}' format: {
		name
			ifNil: [ 'newConfiguration' ]
			ifNotNil: [ name asMCVersionName packageName ].
		Utilities authorInitials.
		name
			ifNil: [ 1 ]
			ifNotNil: [ name asMCVersionName versionNumber + 1 ] }! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/23/2005 00:44'!
summary
	^String streamContents: [:stream |
		self dependencies
			do: [:ea | stream nextPutAll: ea versionInfo name; cr ]]! !

!MCConfiguration methodsFor: 'updating' stamp: 'bf 5/23/2005 17:43'!
updateFromImage
	self dependencies: (self dependencies collect: [:dep |
		dep package hasWorkingCopy
			ifTrue: [
				dep package workingCopy in: [:wc |
					MCVersionDependency package: wc package info: wc ancestors first]]
			ifFalse: [dep]]).
! !

!MCConfiguration methodsFor: 'updating' stamp: 'nice 3/31/2012 22:46'!
updateFromImage: packageIndex
	| dep newDeps |
	dep := self dependencies at: packageIndex.
	newDeps := self dependencies copy.
	newDeps
		at: packageIndex put: (dep package hasWorkingCopy
			ifTrue: [dep package workingCopy in: [:wc |
					MCVersionDependency package: wc package info: wc ancestors first]]
			ifFalse: [dep]).
	self dependencies: newDeps.
! !

!MCConfiguration methodsFor: 'updating' stamp: 'bf 6/22/2012 17:45'!
updateFromRepositories

	self cacheAllFileNamesDuring: [ self updateFromRepositoriesWithoutCaching ]! !

!MCConfiguration methodsFor: 'updating' stamp: 'bf 6/22/2012 17:45'!
updateFromRepositories: packageIndex

	self cacheAllFileNamesDuring: [ self updateFromRepositoriesWithoutCaching: packageIndex ]! !

!MCConfiguration methodsFor: 'updating' stamp: 'bf 2/24/2012 14:43'!
updateFromRepositoriesWithoutCaching

	| oldNames newNames sortedNames newDeps |
	oldNames := self dependencies collect: [:dep | dep versionInfo versionName].
	newNames := Dictionary new.
	self repositories
		do: [:repo | 
			ProgressNotification signal: '' extra: 'Checking ', repo description.
			(repo possiblyNewerVersionsOfAnyOf: oldNames)
				do: [:newName | newNames at: newName put: repo]]
		displayingProgress: 'Searching new versions'.

	sortedNames := newNames keys asArray sort:
		[:a :b | a versionNumber > b versionNumber].

	newDeps := OrderedCollection new: self dependencies size.
	self dependencies
		do: [:dep |
			newDeps add: (sortedNames
				detect: [:each | each packageAndBranchName = dep packageAndBranchName]
				ifFound: [ :newName |
					| repo |
					repo := newNames at: newName.
					(self versionInfoNamed: newName for: dep from: repo)
						ifNil: [ dep ]
						ifNotNil: [ :info |
							MCVersionDependency package: dep package info: info ] ]
				ifNone: [ dep ]) ]
		displayingProgress: 'downloading new versions'.

	self dependencies: newDeps.
! !

!MCConfiguration methodsFor: 'updating' stamp: 'nice 3/31/2012 22:46'!
updateFromRepositoriesWithoutCaching: packageIndex

	| oldNames newNames sortedNames newDeps dep |
	dep := dependencies at: packageIndex.
	oldNames := {dep versionInfo versionName}.
	newNames := Dictionary new.
	self repositories
		do: [:repo | 
			ProgressNotification signal: '' extra: 'Checking ', repo description.
			(repo possiblyNewerVersionsOfAnyOf: oldNames)
				do: [:newName | newNames at: newName put: repo]]
		displayingProgress: 'Searching new versions'.

	sortedNames := newNames keys asArray sort:
		[:a :b | a versionNumber > b versionNumber].

	newDeps := self dependencies copy.
	newDeps at: packageIndex put: (
		sortedNames
				detect: [:each | each packageAndBranchName = dep packageAndBranchName]
				ifFound: [ :newName |
					| repo |
					repo := newNames at: newName.
					(self versionInfoNamed: newName for: dep from: repo)
						ifNil: [ dep ]
						ifNotNil: [ :info |
							MCVersionDependency package: dep package info: info ] ]
				ifNone: [ dep ]).

	self dependencies: newDeps.
! !

!MCConfiguration methodsFor: 'actions' stamp: 'eem 5/15/2015 08:45'!
upgrade
	^self depsSatisfying:
			[:dep | dep isFulfilledByAncestors not]
		versionDo:
			[:ver | 
			(self class upgradeIsMerge and: [ver shouldMerge])
				ifFalse: [ver load]
				ifTrue:
					[[ver merge]
						on: MCNoChangesException
						do: [:req| req resume ]
						on: MCMergeResolutionRequest
						do: [:request |
							request merger conflicts isEmpty
								ifTrue: [request resume: true]
								ifFalse: [request pass]]]]
		displayingProgress: 'upgrading packages'! !

!MCConfiguration methodsFor: 'private' stamp: 'ar 8/6/2009 21:48'!
versionInfoNamed: newName for: dep from: repo
	"Retrieves the version info instead of the version. Searches in-image first, in case the desired version is part of an already loaded package (usual case when doing a partial update). If not present defaults to versionNamed:for:from: an uses its result."
	MCWorkingCopy registry at: dep package ifPresent:[:workingCopy| | seen |
		"Don't use allAncestorsDo: - apparently this can loop indefinitely.
		Rather keep track of the versions that we've seen and make sure we don't loop."
		seen := Set new.
		workingCopy ancestry ancestorsDoWhileTrue:[:vInfo|
			vInfo name = newName ifTrue:[^vInfo].
			(seen includes: vInfo) ifTrue:[false] ifFalse:[seen add: vInfo. false]
		].
	].
	^(self versionNamed: newName for: dep from: repo) info! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 7/5/2012 15:06'!
versionNamed: aMCVersionName for: aDependency from: repo

	| baseName fileName ver |
	(repo filterFileNames: repo cachedFileNames forVersionNamed: aMCVersionName) ifNotEmptyDo: [:cachedNames |
		fileName := cachedNames anyOne.
		self class extraProgressInfo
			ifTrue:[ProgressNotification signal: '' extra: 'Using cached ', fileName].
		ver := repo versionNamed: fileName].
	ver ifNil: [
		baseName := self diffBaseFor: aDependency.
		(baseName notNil and: [baseName ~= aMCVersionName and: [repo includesVersionNamed: baseName]]) ifTrue: [
			fileName := (MCDiffyVersion nameForVer: aMCVersionName base: baseName), '.mcd'.
			self class extraProgressInfo
				ifTrue:[ProgressNotification signal: '' extra: 'Downloading ', fileName].
			ver := [repo versionNamed: fileName] ifError: []]].
	ver ifNil: [
		fileName := aMCVersionName versionName, '.mcz'.
		self class extraProgressInfo
			ifTrue:[ProgressNotification signal: '' extra: 'Downloading ', fileName].
		ver := repo versionNamed: fileName].
	^ver! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 5/4/2013 14:29'!
withProgress: progressString in: aCollection do: aBlock
	^self class extraProgressInfo
		ifTrue: [ aCollection do: aBlock displayingProgress: progressString ]
		ifFalse: [ aCollection do: aBlock ]

! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 10:50'!
writerClass
	^ MCMcmWriter ! !

Object subclass: #MCMcmUpdater
	instanceVariableNames: 'repository updateMapName lastUpdateMap'
	classVariableNames: 'DefaultUpdateMap DefaultUpdateURL Registry SkipPackages UpdateFromServerAtStartup UpdateMissingPackages Updaters'
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCMcmUpdater commentStamp: 'dtl 10/12/2015 19:45' prior: 0!
MCMcmUpdater provides utility methods for updating Monticello packages from Monticello configurations.

When Monticello configurations are stored in a repository (or repositories), MCMcmUpdater acts as an update stream. It first ensures that each configuration map has been loaded in sequence, then updates the last configuration map to the most recent version for each specified package, and finally loads these versions to produce a fully updated configuration.

Currently if a set of packages are unloaded from the image, using this class to reload them may cause problems, depending on what dependencies those classes have.  Success is not assured.  Removing packages via SmalltalkImage>>unloadAllKnownPackages will be successful, it flags the packages removed so that they are not loaded by this utility.

If you wish to not have MCMcmUpdater update packages, there are two ways to handle this:

1) To have MCMcmUpdater not update any packages not currently in the image set the UpdateMissingPackages preference to false:
		MCMcmUpdater updateMissingPackages: false
	Note that any new packages added to the repositories will not be picked up when this is turned off.
2) To have MCMcmUpdater not update a specific package, evaluate
		MCMcmUpdater disableUpdatesOfPackage: <packageName>

Class Variables definitions:

DefaultUpdateURL - String: the URL that will be checked by default for updates.  This would be set for a common standard location to check.

Repository - A registry of known MCMcmUpdater instances identified by repository URL and update map name.

SkipPackages - Set of Strings: names of packages to not update in MCMcmUpdater (empty by default).

UpdateMissingPackages - Boolean: if true (default), new packages in the update config map will be loaded unless they are in SkipPackages.  If false, packages not currently loaded in the image will not be loaded by MCMcmUpdater.  (This can be dangerous if packages are split - use at your own risk).

Instance Variables:

updateMapName - Base name of the files used for this updater, typically a name such as 'update' or 'update.spur'.

repository - URL of the repository in which the update maps are located.

lastUpdateMap - Dictionary of Integer: version number of the last loaded update map per repository.  Keeps track of the last configuration map, so that the utility will not have to run through the full history in the repositories each time you ask to update.
!

!MCMcmUpdater class methodsFor: 'class initialization' stamp: 'dtl 10/12/2015 18:47'!
clearRegistry
	"Save the current default updater, clear the registry, and re-register the current updater.
	This is intended for cleaning up an image prior to public release. Assumes that the
	current updater is the one intended for ongoing use in this image."

	"MCMcmUpdater clearRegistry"

	| current |
	current := self default.
	Registry := nil.
	current register.
	^Registry! !

!MCMcmUpdater class methodsFor: 'instance creation' stamp: 'dtl 10/11/2015 19:45'!
default
	"The default instance for system updates. Uses a default repository and update map
	name that may be set as preferences."

	^self updateMapNamed: self updateMapName repository: self defaultUpdateURL
! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'dtl 5/10/2015 12:32'!
defaultBaseName
	"If not otherwise specified, look for update maps with this base name"

	^ 'update'! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'fbs 12/31/2013 13:59'!
defaultUpdateURL
	"The default update repository URL"

	<preference: 'Update URL'
		category: 'updates'
		description: 'The repository URL for loading updates'
		type: #String>

	^DefaultUpdateURL ifNil:['']! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'ar 4/9/2010 20:40'!
defaultUpdateURL: aString
	"The default update repository URL"

	DefaultUpdateURL := aString! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'cbc 8/25/2010 10:27'!
disableUpdatesOfPackage: packageName
	self skipPackages add: packageName! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'cbc 8/25/2010 10:29'!
enableUpdatesForAllPackages
	SkipPackages := Set new! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'cbc 8/25/2010 10:27'!
enableUpdatesOfPackage: packageName
	self skipPackages remove: packageName ifAbsent: [].! !

!MCMcmUpdater class methodsFor: 'registry' stamp: 'dtl 10/11/2015 19:04'!
forRepository: repository updateMap: basename
	"Answer the requested updater from the repository, or nil of not found"

	"MCMcmUpdater forRepository: 'http://source.squeak.org/trunk' updateMap: 'update'"
	"MCMcmUpdater forRepository: 'foo' updateMap: 'bar'"

	^ ((Registry ifNil: [Registry := Dictionary new])
		at: repository
		ifAbsent: [^nil])
			at: basename
			ifAbsent: [^nil]! !

!MCMcmUpdater class methodsFor: 'class initialization' stamp: 'dtl 10/12/2015 19:34'!
initialize
	"MCMcmUpdater initialize"

	self flag: #TODO. "remove Updaters class var after transition to Registry"

	DefaultUpdateURL ifNil:[
		DefaultUpdateURL := MCHttpRepository trunkUrlString.
		DefaultUpdateMap := self defaultBaseName.
	].

	Registry ifNil: [ "Migrate from Updaters class var to Registry"
		"Set new repository ivar in all existing instances"
		Updaters keysAndValuesDo: [ :k :v | v repository: k].
		"Populate the new registry"
		Updaters do: [:e | e register].
		"Set the default update map name to its prior value"
		self updateMapName:
			((Updaters at: MCMcmUpdater defaultUpdateURL) updateMapName) ].
! !

!MCMcmUpdater class methodsFor: 'registry' stamp: 'dtl 10/13/2015 19:36'!
registry
	"Private - unit test support"
	^Registry! !

!MCMcmUpdater class methodsFor: 'registry' stamp: 'dtl 10/13/2015 19:36'!
registry: registry
	"Private - unit test support"
	Registry := registry! !

!MCMcmUpdater class methodsFor: 'instance creation' stamp: 'dtl 10/11/2015 18:17'!
repository: url updateMap: baseName
	"Answer a new instance with empty last update map, not yet registered"

	^ self repository: url updateMap: baseName lastUpdateMap: Dictionary new! !

!MCMcmUpdater class methodsFor: 'instance creation' stamp: 'dtl 10/11/2015 18:17'!
repository: url updateMap: baseName lastUpdateMap: dictionary
	"Answer a new instance, not yet registered"

	^ self new
		repository: url;
		updateMapName: baseName;
		lastUpdateMap: dictionary! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'cbc 8/25/2010 08:14'!
skipPackages
	^SkipPackages ifNil: [SkipPackages := Set new]! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'dtl 5/2/2015 22:06'!
updateFromRepositories: repositoryUrls
	"MCMcmUpdater updateFromRepositories: #(
		'http://squeaksource.com/MCUpdateTest'
	)"

	^ self default updateFromRepositories: repositoryUrls! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'dtl 5/10/2015 12:32'!
updateFromRepository: updaterUrlKey
	"Update using an MCMcmUpdater identified by updaterUrlKey using the default
	update map baseName"

	^ self updateFromRepository: updaterUrlKey baseName: self defaultBaseName
! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'dtl 10/12/2015 19:23'!
updateFromRepository: updaterUrlKey baseName: baseName
	"Update using an MCMcmUpdater identified by updaterUrlKey, and using
	update map baseName"

	^ (self updateMapNamed: baseName repository: updaterUrlKey)
		doUpdate! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'dtl 10/12/2015 19:23'!
updateFromServer
	"Update the image by loading all pending updates from the server."

	^self default doUpdate
! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'fbs 12/10/2013 17:58'!
updateFromServerAtStartup
	<preference: 'Update from server at startup'
		category: 'updates'
		description: 'If true, the system will check for and load any available updates.'
		type: #Boolean>
	^ UpdateFromServerAtStartup ifNil: [false].! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'fbs 12/10/2013 17:53'!
updateFromServerAtStartup: aBool
	"Whether to update the image on startup."

	UpdateFromServerAtStartup := aBool.! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'dtl 10/11/2015 19:09'!
updateMapName
	"The default update map name"

	<preference: 'Update map name'
		category: 'updates'
		description: 'Base name for the update maps'
		type: #String>

	^DefaultUpdateMap ifNil:['']! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'dtl 10/11/2015 19:10'!
updateMapName: mapName 
	"The default update map name for the default updater."

	DefaultUpdateMap := mapName! !

!MCMcmUpdater class methodsFor: 'instance creation' stamp: 'dtl 10/12/2015 19:06'!
updateMapNamed: baseName repository: url
	"Answer an instance for the given repository URL with a base update name baseName,
	Register a new instance if not present in the registry."

	" | updater1 updater2 |
	updater1 := self updateMapNamed: 'BAR' repository: 'FOO'.
	updater2 := self updateMapNamed: 'BAZ' repository: 'FOO'.
	updater1 unregister.
	updater2 unregister.
	Registry"

	^(self forRepository: url updateMap: baseName)
		ifNil: [ "register a new updater"
			(self repository: url updateMap: baseName) register].

! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'fbs 12/31/2013 14:00'!
updateMissingPackages
	"Whether to update missing (unloaded) packages"

	<preference: 'Update missing package'
		category: 'updates'
		description: 'If true, missing (unloaded) packages will be loaded during the update process.'
		type: #Boolean>

	^UpdateMissingPackages ifNil:[true]! !

!MCMcmUpdater class methodsFor: 'preferences' stamp: 'ar 8/24/2010 20:04'!
updateMissingPackages: aBool
	"Whether to update missing (unloaded) packages"

	UpdateMissingPackages := aBool.! !

!MCMcmUpdater methodsFor: 'updating' stamp: 'dtl 10/24/2015 11:47'!
doUpdate
	"Update the image by loading all pending updates from the server. If this is
	the default updater for the system, update the system version when complete.
	Flush all caches. If a previous download failed this is often helpful"

	^self doUpdate: true
! !

!MCMcmUpdater methodsFor: 'updating' stamp: 'dtl 10/25/2015 17:31'!
doUpdate: interactive
	"Update the image by loading all pending updates from the server. If this is
	the default updater for the system, update the system version when complete.
	If interteractive use a modal notifier, otherwise only update the transcript.
	Flush all caches. If a previous download failed this is often helpful"

	| config |
	MCFileBasedRepository flushAllCaches.
	config := self updateFromRepositories: { self repository }.
	config ifNil: [
		interactive ifTrue: [ ^self inform: 'Unable to retrieve updates from remote repository.' translated ].
		Transcript cr; show: '==========  Unable to retrieve updates from remote repository. ==========' translated; cr.
		^ self ].
	MCMcmUpdater default == self
		ifTrue: [	config setSystemVersion.
			interactive ifTrue: [ self inform: ('Update completed.
Current update number: ' translated, SystemVersion current highestUpdate) ].
			Transcript cr;
				show: '==========  Update completed. Current update number ' translated;
				show: SystemVersion current highestUpdate;
				show: ' =========='; cr ]
		ifFalse: [
			interactive
				ifTrue: [ self inform: 'Update completed.' ].
			Transcript cr; show: '==========  Update completed. ==========' translated; cr ]
	! !

!MCMcmUpdater methodsFor: 'registry' stamp: 'dtl 10/11/2015 18:26'!
isRegistered
	"True if this instance is registered. False if another instance with the same
	repository and updateNameName is registered."
	
	^self == ((Registry
		at: repository
		ifAbsent: [^false])
			at: updateMapName
			ifAbsent: [^false]).

! !

!MCMcmUpdater methodsFor: 'accessing' stamp: 'dtl 5/2/2015 21:34'!
lastUpdateMap

	^ lastUpdateMap ifNil: [ lastUpdateMap := Dictionary new ]
! !

!MCMcmUpdater methodsFor: 'accessing' stamp: 'dtl 5/2/2015 22:13'!
lastUpdateMap: aDictionary

	lastUpdateMap := aDictionary
! !

!MCMcmUpdater methodsFor: 'printing' stamp: 'dtl 10/11/2015 22:57'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: ' on ''';
		nextPutAll: updateMapName asString;
		nextPutAll:  ''' at ';
		nextPutAll: repository asString! !

!MCMcmUpdater methodsFor: 'updating' stamp: 'dtl 5/10/2015 19:32'!
refreshUpdateMapFor: r with: updateList
	"Update the lastUpdateMap and answer a possibly reduced updateList"

	| config |
	(lastUpdateMap at: r description ifAbsent: [0]) = 0 ifTrue: [
		"No update has ever been loaded from this repo. If no package is
		present in the image either, we can skip right to the latest config"
		config := r versionNamed: updateList last value.
		(config dependencies anySatisfy: [:dep | dep package hasWorkingCopy])
			ifFalse: [(self useLatestPackagesFrom: r)
					ifTrue: [lastUpdateMap at: r description put: updateList last key].
				updateList isEmpty
					ifTrue: [^ #()]
					ifFalse: [^ updateList last: 1]]].
	^ updateList
! !

!MCMcmUpdater methodsFor: 'registry' stamp: 'dtl 10/11/2015 19:18'!
register
	"Register this instance, keyed by repository and update map name. Each update
	 maintains its own lastUpdateMap. The registry permits multilple updaters to be
	maintained, with each updater keeping track of its own last update map."
	
	repository ifNil: [self error: 'repository is ', repository asString].
	updateMapName ifNil: [self error: 'updateMapName is ', updateMapName asString].
	updateMapName isEmpty ifTrue:  [self error: 'updateMapName must be specified'].
	((Registry ifNil: [Registry := Dictionary new])
		at: repository
		ifAbsentPut: [Dictionary new])
			at: updateMapName put: self

! !

!MCMcmUpdater methodsFor: 'accessing' stamp: 'dtl 10/11/2015 16:24'!
repository
	"URL string of the repository for the update maps"

	^ repository! !

!MCMcmUpdater methodsFor: 'accessing' stamp: 'dtl 10/11/2015 16:23'!
repository: repositoryURLString

	repository := repositoryURLString! !

!MCMcmUpdater methodsFor: 'private' stamp: 'cbc 8/25/2010 08:14'!
skipPackages
	^SkipPackages ifNil: [SkipPackages := Set new]! !

!MCMcmUpdater methodsFor: 'registry' stamp: 'dtl 10/12/2015 19:00'!
unregister
	"If this instance is registered, remove it frorm the registry."
	
	self isRegistered
		ifTrue: [(Registry at: repository) removeKey: updateMapName.
				(Registry at: repository) isEmpty
					ifTrue: [Registry removeKey: repository]]
! !

!MCMcmUpdater methodsFor: 'updating' stamp: 'dtl 5/2/2015 22:26'!
updateFromConfig: config

	"Skip packages that were specifically unloaded"
	config dependencies: (config dependencies 
		reject: [:dep| self class skipPackages includes: dep package name]).
	self class updateMissingPackages ifFalse:[
		"Skip packages that are not in the image"
		config dependencies: (config dependencies 
			select: [:dep| dep package hasWorkingCopy])].
	(config dependencies allSatisfy:[:dep| dep isFulfilled]) 
		ifFalse:[config upgrade].
! !

!MCMcmUpdater methodsFor: 'updating' stamp: 'dtl 5/2/2015 21:36'!
updateFromRepositories: repositoryUrls
	"MCMcmUpdater updateFromRepositories: #(
		'http://squeaksource.com/MCUpdateTest'
	)"

	| repos config |
	MCConfiguration upgradeIsMerge: true.
	"The list of repositories to consult in order"
	repos := repositoryUrls collect:[:url| 
		MCRepositoryGroup default repositories 
			detect:[:r| r description = url]
			ifNone:[ | r |
				r := MCHttpRepository location: url user: '' password: ''.
				MCRepositoryGroup default addRepository: r.
				r]].

	"The list of updates-author.version.mcm sorted by version"
	repos do:[ :r | config := self updateFromRepository: r ].
	^config! !

!MCMcmUpdater methodsFor: 'updating' stamp: 'dtl 5/2/2015 21:36'!
updateFromRepository: repository

	| config |
	repository cacheAllFileNamesDuring: [ | updateList |
		updateList := self updateListFor: repository.
		"Proceed only if there are updates available at all."
		updateList ifNotEmpty: [
			updateList := self refreshUpdateMapFor: repository with: updateList.
			"Now process each update file. Check if we have all dependencies and if not,
			load the entire configuration (this is mostly to skip older updates quickly)"
			updateList do:[:assoc|
				ProgressNotification signal: '' extra: 'Processing ', assoc value.
				config := repository versionNamed: assoc value.
				self updateFromConfig: config.
				self lastUpdateMap at: repository description put: assoc key.
			] displayingProgress: 'Processing configurations'.
			"We've loaded all the provided update configurations.
			Use the latest configuration to update all the remaining packages."
			(self useLatestPackagesFrom: repository) ifTrue: [
				config updateFromRepositories.
				config upgrade].
		]].
	^ config
! !

!MCMcmUpdater methodsFor: 'private' stamp: 'dtl 5/2/2015 21:36'!
updateListFor: repo

	| updateList allNames minVersion |
	updateList := OrderedCollection new.
	minVersion := self lastUpdateMap at: repo description ifAbsent: [0].
	"Find all the update-*.mcm files"
	allNames := 'Checking ', repo description
		displayProgressFrom: 0 to: 1 during: [:bar| 
			bar value: 0.
			repo allFileNamesOrCache ].
	allNames do: [:fileName | | version |
		((fileName endsWith: '.mcm')
			and: [fileName packageAndBranchName = self updateMapName
				and: [(version := fileName versionNumber) >= minVersion]]) 
					ifTrue: [updateList add: version -> fileName]].
	^updateList sort! !

!MCMcmUpdater methodsFor: 'accessing' stamp: 'dtl 5/2/2015 21:34'!
updateMapName
	"Name for update map, without version info"

	^ updateMapName ifNil: [updateMapName := self class updateMapName]! !

!MCMcmUpdater methodsFor: 'accessing' stamp: 'dtl 5/2/2015 21:15'!
updateMapName: aString
	"Name for update map, without version info"
	updateMapName := aString! !

!MCMcmUpdater methodsFor: 'private' stamp: 'bf 7/12/2011 19:25'!
useLatestPackagesFrom: repo
	"for overriding on a per repository basis"
	^true! !

!MCFileBasedRepository class methodsFor: '*monticelloconfigurations' stamp: 'cmm 4/22/2011 16:51'!
supportsConfigurations
	^ true! !

MCVersionReader subclass: #MCMcmReader
	instanceVariableNames: 'fileName configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCMcmReader commentStamp: 'dtl 5/10/2010 22:22' prior: 0!
A MCMcmReader creates an MCConfiguration by reading an array specification from a stream.

!

!MCMcmReader class methodsFor: 'accessing' stamp: 'bf 3/22/2005 10:47'!
extension
	^ 'mcm'! !

!MCMcmReader class methodsFor: 'instance creation' stamp: 'ar 9/19/2009 11:49'!
loadVersionFile: fileName
	| version |
	version := self versionFromFile: fileName.
	version load.
! !

!MCMcmReader class methodsFor: 'instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
on: aStream fileName: aFileName
	| reader |
	reader := self on: aStream.
	reader fileName: aFileName.
	^reader! !

!MCMcmReader methodsFor: 'accessing' stamp: 'ar 6/30/2009 17:05'!
configuration
	configuration ifNil: [self loadConfiguration].
	"browser modifies configuration, but the reader might get cached"
	^configuration copy! !

!MCMcmReader methodsFor: 'accessing' stamp: 'bf 3/23/2005 01:17'!
fileName: aString
	fileName := aString! !

!MCMcmReader methodsFor: 'accessing' stamp: 'cmm 10/8/2013 10:58'!
loadConfiguration
	stream reset.
	configuration := MCConfiguration fromArray: (MCScanner scan: stream).
	configuration name ifNil: [ configuration name: self parseNameFromFilename ]! !

!MCMcmReader methodsFor: 'accessing' stamp: 'ar 6/30/2009 17:06'!
loadVersionInfo
	info := self configuration! !

!MCMcmReader methodsFor: 'accessing' stamp: 'cmm 10/8/2013 10:58'!
parseNameFromFilename
	^fileName ifNotNil: [(fileName findTokens: '/\:') last copyUpToLast: $.]! !

!MCMcmReader methodsFor: 'accessing' stamp: 'ar 6/30/2009 17:06'!
version
	^self configuration! !
MCConfigurationBrowser initialize!
MCMcmUpdater initialize!
